/*!
 * type-reverse <https://github.com/whizkydee/type-reverse>
 *
 * Copyright (c) 2017-present, Olaolu Olawuyi.
 * Released under the MIT License.
 */

'use strict';

/**
 * 🦄 Lightweight reverse utility around strings, arrays, numbers and more.
 *
 * @name reverse
 * @alias inverse
 * @param {String|Number|Array|Set|NodeList} `input`
 * @param {?Object} `options`
 * @return {*}
 * @api public
 */

Object.defineProperty(exports, "__esModule", {
  value: !0
});

var _util = require('./util');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } return Array.from(arr); }

function reverse(input) {
  var options = 1 < arguments.length && arguments[1] !== undefined ? arguments[1] : {},
      globArr = [].concat(_toConsumableArray(input)).reverse(),
      then = options.then,
      minusRE = /^-/;


  if (input && !(0, _util.supported)(input)) throw new TypeError(_util.errprefix + ' ' + (0, _util._typeof)(input) + 's are not supported');

  options.invert = options.invert || 'index', options.then = then || function (_, v) {
    return v;
  };


  var result = void 0;
  switch ((0, _util._typeof)(input)) {
    case 'string':
      switch (options.invert) {
        case 'index':
          result = globArr.join('');
          break;
        case 'word':
          result = input.split(' ').reverse().join(' ');

      }
      break;

    case 'number':
      // convert the number to string then replace the minus(-) symbol with nothing
      var nStr = String(input).replace(minusRE, '');

      if (/e/.test(nStr)) throw new TypeError('Oops. That number is too large. See https://github.com/whizkydee/type-reverse/blob/master/readme.md#limits for more info.');

      switch (options.invert) {
        case 'sign':
          result = minusRE.test(input) ? Number(+nStr) : Number(-nStr);

          break;
        case 'index':
          result = minusRE.test(input) ? reverse(nStr, { then: function (_, x) {
              return Number(-x);
            } }) : reverse(nStr, { then: function (_, x) {
              return Number(x);
            } });

      }
      break;

    case 'array':case 'nodelist':
      result = globArr;
      break;

    case 'set':
      result = new Set(globArr);
      break;

    default:
      result = reverse(input);

  }

  if ('function' === typeof then) return then.call(this, input, result);
  if (then && 'function' !== typeof then) throw new TypeError(_util.errprefix + ' Expected function as second argument, got ' + (0, _util._typeof)(then) + '.');
  return result;
}

exports.default = reverse;
module.exports = reverse;